@@ -1,19 +1,19 @@
-from functools import partial
 import numpy as np
 import theano
 import theano.tensor as tt
 from scipy import stats
+import warnings
 
 from pymc3.util import get_variable_name
-from .dist_math import bound, factln, binomln, betaln, logpow
-from .distribution import Discrete, draw_values, generate_samples, reshape_sampled
-from pymc3.math import tround
-from ..math import logaddexp
+from .dist_math import bound, factln, binomln, betaln, logpow, random_choice
+from .distribution import Discrete, draw_values, generate_samples
+from pymc3.math import tround, sigmoid, logaddexp, logit, log1pexp
+
 
 __all__ = ['Binomial',  'BetaBinomial',  'Bernoulli',  'DiscreteWeibull',
            'Poisson', 'NegativeBinomial', 'ConstantDist', 'Constant',
            'ZeroInflatedPoisson', 'ZeroInflatedBinomial', 'ZeroInflatedNegativeBinomial',
-           'DiscreteUniform', 'Geometric', 'Categorical']
+           'DiscreteUniform', 'Geometric', 'Categorical', 'OrderedLogistic']
 
 
 class Binomial(Discrete):
@@ -23,9 +23,27 @@ class Binomial(Discrete):
     The discrete probability distribution of the number of successes
     in a sequence of n independent yes/no experiments, each of which
     yields success with probability p.
+    The pmf of this distribution is
 
     .. math:: f(x \mid n, p) = \binom{n}{x} p^x (1-p)^{n-x}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = np.arange(0, 22)
+        ns = [10, 17]
+        ps = [0.5, 0.7]
+        for n, p in zip(ns, ps):
+            pmf = st.binom.pmf(x, n, p)
+            plt.plot(x, pmf, '-o', label='n = {}, p = {}'.format(n, p))
+        plt.xlabel('x', fontsize=14)
+        plt.ylabel('f(x)', fontsize=14)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ==========================================
     Support   :math:`x \in \{0, 1, \ldots, n\}`
     Mean      :math:`n p`
@@ -46,8 +64,8 @@ class Binomial(Discrete):
         self.p = p = tt.as_tensor_variable(p)
         self.mode = tt.cast(tround(n * p), self.dtype)
 
-    def random(self, point=None, size=None, repeat=None):
-        n, p = draw_values([self.n, self.p], point=point)
+    def random(self, point=None, size=None):
+        n, p = draw_values([self.n, self.p], point=point, size=size)
         return generate_samples(stats.binom.rvs, n=n, p=p,
                                 dist_shape=self.shape,
                                 size=size)
@@ -66,7 +84,8 @@ class Binomial(Discrete):
             dist = self
         n = dist.n
         p = dist.p
-        return r'${} \sim \text{{Binomial}}(\mathit{{n}}={}, \mathit{{p}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{Binomial}}(\mathit{{n}}={},~\mathit{{p}}={})$'.format(name,
                                                 get_variable_name(n),
                                                 get_variable_name(p))
 
@@ -76,6 +95,7 @@ class BetaBinomial(Discrete):
 
     Equivalent to binomial random variable with success probability
     drawn from a beta distribution.
+    The pmf of this distribution is
 
     .. math::
 
@@ -83,6 +103,31 @@ class BetaBinomial(Discrete):
            \binom{n}{x}
            \frac{B(x + \alpha, n - x + \beta)}{B(\alpha, \beta)}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        from scipy import special
+        plt.style.use('seaborn-darkgrid')
+
+        def BetaBinom(a, b, n, x):
+            pmf = special.binom(n, x) * (special.beta(x+a, n-x+b) / special.beta(a, b))
+            return pmf
+
+        x = np.arange(0, 11)
+        alphas = [0.5, 1, 2.3]
+        betas = [0.5, 1, 2]
+        n = 10
+        for a, b in zip(alphas, betas):
+            pmf = BetaBinom(a, b, n, x)
+            plt.plot(x, pmf, '-o', label=r'$\alpha$ = {}, $\beta$ = {}, n = {}'.format(a, b, n))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0)
+        plt.legend(loc=9)
+        plt.show()
+
     ========  =================================================================
     Support   :math:`x \in \{0, 1, \ldots, n\}`
     Mean      :math:`n \dfrac{\alpha}{\alpha + \beta}`
@@ -108,19 +153,26 @@ class BetaBinomial(Discrete):
 
     def _random(self, alpha, beta, n, size=None):
         size = size or 1
-        p = np.atleast_1d(stats.beta.rvs(a=alpha, b=beta, size=np.prod(size)))
+        p = stats.beta.rvs(a=alpha, b=beta, size=size).flatten()
         # Sometimes scipy.beta returns nan. Ugh.
         while np.any(np.isnan(p)):
             i = np.isnan(p)
             p[i] = stats.beta.rvs(a=alpha, b=beta, size=np.sum(i))
         # Sigh...
-        _n, _p, _size = np.atleast_1d(n).flatten(), p.flatten(), np.prod(size)
+        _n, _p, _size = np.atleast_1d(n).flatten(), p.flatten(), p.shape[0]
+
+        quotient, remainder = divmod(_p.shape[0], _n.shape[0])
+        if remainder != 0:
+            raise TypeError('n has a bad size! Was cast to {}, must evenly divide {}'.format(
+                _n.shape[0], _p.shape[0]))
+        if quotient != 1:
+            _n = np.tile(_n, quotient)
         samples = np.reshape(stats.binom.rvs(n=_n, p=_p, size=_size), size)
         return samples
 
-    def random(self, point=None, size=None, repeat=None):
+    def random(self, point=None, size=None):
         alpha, beta, n = \
-            draw_values([self.alpha, self.beta, self.n], point=point)
+            draw_values([self.alpha, self.beta, self.n], point=point, size=size)
         return generate_samples(self._random, alpha=alpha, beta=beta, n=n,
                                 dist_shape=self.shape,
                                 size=size)
@@ -139,7 +191,8 @@ class BetaBinomial(Discrete):
             dist = self
         alpha = dist.alpha
         beta = dist.beta
-        return r'${} \sim \text{{NegativeBinomial}}(\mathit{{alpha}}={}, \mathit{{beta}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{BetaBinomial}}(\mathit{{alpha}}={},~\mathit{{beta}}={})$'.format(name,
                                                 get_variable_name(alpha),
                                                 get_variable_name(beta))
 
@@ -149,9 +202,26 @@ class Bernoulli(Discrete):
 
     The Bernoulli distribution describes the probability of successes
     (x=1) and failures (x=0).
+    The pmf of this distribution is
 
     .. math:: f(x \mid p) = p^{x} (1-p)^{1-x}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = [0, 1]
+        for p in [0, 0.5, 0.8]:
+            pmf = st.bernoulli.pmf(x, p)
+            plt.plot(x, pmf, '-o', label='p = {}'.format(p))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0)
+        plt.legend(loc=9)
+        plt.show()
+
     ========  ======================
     Support   :math:`x \in \{0, 1\}`
     Mean      :math:`p`
@@ -162,30 +232,48 @@ class Bernoulli(Discrete):
     ----------
     p : float
         Probability of success (0 < p < 1).
+    logit_p : float
+        Logit of success probability. Only one of `p` and `logit_p`
+        can be specified.
     """
 
-    def __init__(self, p, *args, **kwargs):
+    def __init__(self, p=None, logit_p=None, *args, **kwargs):
         super(Bernoulli, self).__init__(*args, **kwargs)
-        self.p = p = tt.as_tensor_variable(p)
-        self.mode = tt.cast(tround(p), 'int8')
+        if sum(int(var is None) for var in [p, logit_p]) != 1:
+            raise ValueError('Specify one of p and logit_p')
+        if p is not None:
+            self._is_logit = False
+            self.p = p = tt.as_tensor_variable(p)
+            self._logit_p = logit(p)
+        else:
+            self._is_logit = True
+            self.p = tt.nnet.sigmoid(logit_p)
+            self._logit_p = tt.as_tensor_variable(logit_p)
 
-    def random(self, point=None, size=None, repeat=None):
-        p = draw_values([self.p], point=point)[0]
+        self.mode = tt.cast(tround(self.p), 'int8')
+
+    def random(self, point=None, size=None):
+        p = draw_values([self.p], point=point, size=size)[0]
         return generate_samples(stats.bernoulli.rvs, p,
                                 dist_shape=self.shape,
                                 size=size)
 
     def logp(self, value):
-        p = self.p
-        return bound(
-            tt.switch(value, tt.log(p), tt.log(1 - p)),
-            value >= 0, value <= 1,
-            p >= 0, p <= 1)
+        if self._is_logit:
+            lp = tt.switch(value, self._logit_p, -self._logit_p)
+            return -log1pexp(-lp)
+        else:
+            p = self.p
+            return bound(
+                tt.switch(value, tt.log(p), tt.log(1 - p)),
+                value >= 0, value <= 1,
+                p >= 0, p <= 1)
 
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
             dist = self
         p = dist.p
+        name = r'\text{%s}' % name
         return r'${} \sim \text{{Bernoulli}}(\mathit{{p}}={})$'.format(name,
                                                 get_variable_name(p))
 
@@ -195,9 +283,33 @@ class DiscreteWeibull(Discrete):
 
     The discrete Weibull distribution is a flexible model of count data that
     can handle both over- and under-dispersion.
+    The pmf of this distribution is
 
     .. math:: f(x \mid q, \beta) = q^{x^{\beta}} - q^{(x + 1)^{\beta}}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        from scipy import special
+        plt.style.use('seaborn-darkgrid')
+
+        def DiscreteWeibull(q, b, x):
+            return q**(x**b) - q**((x + 1)**b)
+
+        x = np.arange(0, 10)
+        qs = [0.1, 0.9, 0.9]
+        betas = [0.3, 1.3, 3]
+        for q, b in zip(qs, betas):
+            pmf = DiscreteWeibull(q, b, x)
+            plt.plot(x, pmf, '-o', label=r'q = {}, $\beta$ = {}'.format(q, b))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ======================
     Support   :math:`x \in \mathbb{N}_0`
     Mean      :math:`\mu = \sum_{x = 1}^{\infty} q^{x^{\beta}}`
@@ -205,7 +317,7 @@ class DiscreteWeibull(Discrete):
     ========  ======================
     """
     def __init__(self, q, beta, *args, **kwargs):
-        super(DiscreteWeibull, self).__init__(*args, defaults=['median'], **kwargs)
+        super(DiscreteWeibull, self).__init__(*args, defaults=('median',), **kwargs)
 
         self.q = q = tt.as_tensor_variable(q)
         self.beta = beta = tt.as_tensor_variable(beta)
@@ -236,8 +348,8 @@ class DiscreteWeibull(Discrete):
 
         return np.ceil(np.power(np.log(1 - p) / np.log(q), 1. / beta)) - 1
 
-    def random(self, point=None, size=None, repeat=None):
-        q, beta = draw_values([self.q, self.beta], point=point)
+    def random(self, point=None, size=None):
+        q, beta = draw_values([self.q, self.beta], point=point, size=size)
 
         return generate_samples(self._random, q, beta,
                                 dist_shape=self.shape,
@@ -248,7 +360,8 @@ class DiscreteWeibull(Discrete):
             dist = self
         q = dist.q
         beta = dist.beta
-        return r'${} \sim \text{{DiscreteWeibull}}(\mathit{{q}}={}, \mathit{{beta}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{DiscreteWeibull}}(\mathit{{q}}={},~\mathit{{beta}}={})$'.format(name,
                                                 get_variable_name(q),
                                                 get_variable_name(beta))
 
@@ -259,9 +372,26 @@ class Poisson(Discrete):
 
     Often used to model the number of events occurring in a fixed period
     of time when the times at which events occur are independent.
+    The pmf of this distribution is
 
     .. math:: f(x \mid \mu) = \frac{e^{-\mu}\mu^x}{x!}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = np.arange(0, 15)
+        for m in [0.5, 3, 8]:
+            pmf = st.poisson.pmf(x, m)
+            plt.plot(x, pmf, '-o', label='$\mu$ = {}'.format(m))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ==========================
     Support   :math:`x \in \mathbb{N}_0`
     Mean      :math:`\mu`
@@ -285,8 +415,8 @@ class Poisson(Discrete):
         self.mu = mu = tt.as_tensor_variable(mu)
         self.mode = tt.floor(mu).astype('int32')
 
-    def random(self, point=None, size=None, repeat=None):
-        mu = draw_values([self.mu], point=point)[0]
+    def random(self, point=None, size=None):
+        mu = draw_values([self.mu], point=point, size=size)[0]
         return generate_samples(stats.poisson.rvs, mu,
                                 dist_shape=self.shape,
                                 size=size)
@@ -297,13 +427,14 @@ class Poisson(Discrete):
             logpow(mu, value) - factln(value) - mu,
             mu >= 0, value >= 0)
         # Return zero when mu and value are both zero
-        return tt.switch(1 * tt.eq(mu, 0) * tt.eq(value, 0),
+        return tt.switch(tt.eq(mu, 0) * tt.eq(value, 0),
                          0, log_prob)
 
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
             dist = self
         mu = dist.mu
+        name = r'\text{%s}' % name
         return r'${} \sim \text{{Poisson}}(\mathit{{mu}}={})$'.format(name,
                                                 get_variable_name(mu))
 
@@ -314,13 +445,37 @@ class NegativeBinomial(Discrete):
 
     The negative binomial distribution describes a Poisson random variable
     whose rate parameter is gamma distributed.
+    The pmf of this distribution is
 
     .. math::
 
        f(x \mid \mu, \alpha) =
-           \frac{\Gamma(x+\alpha)}{x! \Gamma(\alpha)}
+           \binom{x + \alpha - 1}{x}
            (\alpha/(\mu+\alpha))^\alpha (\mu/(\mu+\alpha))^x
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        from scipy import special
+        plt.style.use('seaborn-darkgrid')
+
+        def NegBinom(a, m, x):
+            pmf = special.binom(x + a - 1, x) * (a / (m + a))**a * (m / (m + a))**x
+            return pmf
+
+        x = np.arange(0, 22)
+        alphas = [0.9, 2, 4]
+        mus = [1, 2, 8]
+        for a, m in zip(alphas, mus):
+            pmf = NegBinom(a, m, x)
+            plt.plot(x, pmf, '-o', label=r'$\alpha$ = {}, $\mu$ = {}'.format(a, m))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ==========================
     Support   :math:`x \in \mathbb{N}_0`
     Mean      :math:`\mu`
@@ -340,13 +495,13 @@ class NegativeBinomial(Discrete):
         self.alpha = alpha = tt.as_tensor_variable(alpha)
         self.mode = tt.floor(mu).astype('int32')
 
-    def random(self, point=None, size=None, repeat=None):
-        mu, alpha = draw_values([self.mu, self.alpha], point=point)
+    def random(self, point=None, size=None):
+        mu, alpha = draw_values([self.mu, self.alpha], point=point, size=size)
         g = generate_samples(stats.gamma.rvs, alpha, scale=mu / alpha,
                              dist_shape=self.shape,
                              size=size)
         g[g == 0] = np.finfo(float).eps  # Just in case
-        return reshape_sampled(stats.poisson.rvs(g), size, self.shape)
+        return np.asarray(stats.poisson.rvs(g)).reshape(g.shape)
 
     def logp(self, value):
         mu = self.mu
@@ -357,7 +512,7 @@ class NegativeBinomial(Discrete):
                          value >= 0, mu > 0, alpha > 0)
 
         # Return Poisson when alpha gets very large.
-        return tt.switch(1 * (alpha > 1e10),
+        return tt.switch(tt.gt(alpha, 1e10),
                          Poisson.dist(self.mu).logp(value),
                          negbinom)
 
@@ -366,7 +521,8 @@ class NegativeBinomial(Discrete):
             dist = self
         mu = dist.mu
         alpha = dist.alpha
-        return r'${} \sim \text{{NegativeBinomial}}(\mathit{{mu}}={}, \mathit{{alpha}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{NegativeBinomial}}(\mathit{{mu}}={},~\mathit{{alpha}}={})$'.format(name,
                                                 get_variable_name(mu),
                                                 get_variable_name(alpha))
 
@@ -377,9 +533,25 @@ class Geometric(Discrete):
 
     The probability that the first success in a sequence of Bernoulli
     trials occurs on the x'th trial.
+    The pmf of this distribution is
 
     .. math:: f(x \mid p) = p(1-p)^{x-1}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = np.arange(1, 11)
+        for p in [0.1, 0.25, 0.75]:
+            pmf = st.geom.pmf(x, p)
+            plt.plot(x, pmf, '-o', label='p = {}'.format(p))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  =============================
     Support   :math:`x \in \mathbb{N}_{>0}`
     Mean      :math:`\dfrac{1}{p}`
@@ -397,8 +569,8 @@ class Geometric(Discrete):
         self.p = p = tt.as_tensor_variable(p)
         self.mode = 1
 
-    def random(self, point=None, size=None, repeat=None):
-        p = draw_values([self.p], point=point)[0]
+    def random(self, point=None, size=None):
+        p = draw_values([self.p], point=point, size=size)[0]
         return generate_samples(np.random.geometric, p,
                                 dist_shape=self.shape,
                                 size=size)
@@ -412,6 +584,7 @@ class Geometric(Discrete):
         if dist is None:
             dist = self
         p = dist.p
+        name = r'\text{%s}' % name
         return r'${} \sim \text{{Geometric}}(\mathit{{p}}={})$'.format(name,
                                                 get_variable_name(p))
 
@@ -419,9 +592,28 @@ class Geometric(Discrete):
 class DiscreteUniform(Discrete):
     R"""
     Discrete uniform distribution.
+    The pmf of this distribution is
 
     .. math:: f(x \mid lower, upper) = \frac{1}{upper-lower}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        ls = [1, -2]
+        us = [6, 2]
+        for l, u in zip(ls, us):
+            x = np.arange(l, u+1)
+            pmf = [1 / (u - l)] * len(x)
+            plt.plot(x, pmf, '-o', label='lower = {}, upper = {}'.format(l, u))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0, 0.4)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ===============================================
     Support   :math:`x \in {lower, lower + 1, \ldots, upper}`
     Mean      :math:`\dfrac{lower + upper}{2}`
@@ -446,12 +638,11 @@ class DiscreteUniform(Discrete):
     def _random(self, lower, upper, size=None):
         # This way seems to be the only to deal with lower and upper
         # as array-like.
-        samples = stats.uniform.rvs(lower, upper - lower - np.finfo(float).eps,
-                                    size=size)
-        return np.floor(samples).astype('int32')
+        samples = stats.randint.rvs(lower, upper + 1, size=size)
+        return samples
 
-    def random(self, point=None, size=None, repeat=None):
-        lower, upper = draw_values([self.lower, self.upper], point=point)
+    def random(self, point=None, size=None):
+        lower, upper = draw_values([self.lower, self.upper], point=point, size=size)
         return generate_samples(self._random,
                                 lower, upper,
                                 dist_shape=self.shape,
@@ -468,7 +659,8 @@ class DiscreteUniform(Discrete):
             dist = self
         lower = dist.lower
         upper = dist.upper
-        return r'${} \sim \text{{DiscreteUniform}}(\mathit{{lower}}={}, \mathit{{upper}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{DiscreteUniform}}(\mathit{{lower}}={},~\mathit{{upper}}={})$'.format(name,
                                                 get_variable_name(lower),
                                                 get_variable_name(upper))
 
@@ -477,12 +669,28 @@ class Categorical(Discrete):
     R"""
     Categorical log-likelihood.
 
-    The most general discrete distribution.
+    The most general discrete distribution. The pmf of this distribution is
 
     .. math:: f(x \mid p) = p_x
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        ps = [[0.1, 0.6, 0.3], [0.3, 0.1, 0.1, 0.5]]
+        for p in ps:
+            x = range(len(p))
+            plt.plot(x, p, '-o', label='p = {}'.format(p))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.ylim(0)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ===================================
-    Support   :math:`x \in \{1, 2, \ldots, |p|\}`
+    Support   :math:`x \in \{0, 1, \ldots, |p|-1\}`
     ========  ===================================
 
     Parameters
@@ -498,22 +706,14 @@ class Categorical(Discrete):
             self.k = tt.shape(p)[-1].tag.test_value
         except AttributeError:
             self.k = tt.shape(p)[-1]
-        self.p = p = tt.as_tensor_variable(p)
+        p = tt.as_tensor_variable(p)
         self.p = (p.T / tt.sum(p, -1)).T
         self.mode = tt.argmax(p)
 
-    def random(self, point=None, size=None, repeat=None):
-        def random_choice(k, *args, **kwargs):
-            if len(kwargs['p'].shape) > 1:
-                return np.asarray(
-                    [np.random.choice(k, p=p)
-                     for p in kwargs['p']]
-                )
-            else:
-                return np.random.choice(k, *args, **kwargs)
+    def random(self, point=None, size=None):
+        p, k = draw_values([self.p, self.k], point=point, size=size)
 
-        p, k = draw_values([self.p, self.k], point=point)
-        return generate_samples(partial(random_choice, np.arange(k)),
+        return generate_samples(random_choice,
                                 p=p,
                                 broadcast_shape=p.shape[:-1] or (1,),
                                 dist_shape=self.shape,
@@ -540,6 +740,7 @@ class Categorical(Discrete):
         if dist is None:
             dist = self
         p = dist.p
+        name = r'\text{%s}' % name
         return r'${} \sim \text{{Categorical}}(\mathit{{p}}={})$'.format(name,
                                                 get_variable_name(p))
 
@@ -555,11 +756,13 @@ class Constant(Discrete):
     """
 
     def __init__(self, c, *args, **kwargs):
+        warnings.warn("Constant has been deprecated. We recommend using a Deterministic object instead.",
+                    DeprecationWarning)
         super(Constant, self).__init__(*args, **kwargs)
         self.mean = self.median = self.mode = self.c = c = tt.as_tensor_variable(c)
 
-    def random(self, point=None, size=None, repeat=None):
-        c = draw_values([self.c], point=point)[0]
+    def random(self, point=None, size=None):
+        c = draw_values([self.c], point=point, size=size)[0]
         dtype = np.array(c).dtype
 
         def _random(c, dtype=dtype, size=None):
@@ -575,14 +778,11 @@ class Constant(Discrete):
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
             dist = self
+        name = r'\text{%s}' % name
         return r'${} \sim \text{{Constant}}()$'.format(name)
 
 
-def ConstantDist(*args, **kwargs):
-    import warnings
-    warnings.warn("ConstantDist has been deprecated. In future, use Constant instead.",
-                  DeprecationWarning)
-    return Constant(*args, **kwargs)
+ConstantDist = Constant
 
 
 class ZeroInflatedPoisson(Discrete):
@@ -591,6 +791,7 @@ class ZeroInflatedPoisson(Discrete):
 
     Often used to model the number of events occurring in a fixed period
     of time when the times at which events occur are independent.
+    The pmf of this distribution is
 
     .. math::
 
@@ -599,6 +800,26 @@ class ZeroInflatedPoisson(Discrete):
             \psi \frac{e^{-\theta}\theta^x}{x!}, \text{if } x=1,2,3,\ldots
             \end{array} \right.
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = np.arange(0, 22)
+        psis = [0.7, 0.4]
+        thetas = [8, 4]
+        for psi, theta in zip(psis, thetas):
+            pmf = st.poisson.pmf(x, theta)
+            pmf[0] = (1 - psi) + pmf[0]
+            pmf[1:] =  psi * pmf[1:]
+            pmf /= pmf.sum()
+            plt.plot(x, pmf, '-o', label='$\\psi$ = {}, $\\theta$ = {}'.format(psi, theta))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ==========================
     Support   :math:`x \in \mathbb{N}_0`
     Mean      :math:`\psi\theta`
@@ -621,23 +842,24 @@ class ZeroInflatedPoisson(Discrete):
         self.pois = Poisson.dist(theta)
         self.mode = self.pois.mode
 
-    def random(self, point=None, size=None, repeat=None):
-        theta, psi = draw_values([self.theta, self.psi], point=point)
+    def random(self, point=None, size=None):
+        theta, psi = draw_values([self.theta, self.psi], point=point, size=size)
         g = generate_samples(stats.poisson.rvs, theta,
                              dist_shape=self.shape,
                              size=size)
-        sampled = g * (np.random.random(np.squeeze(g.shape)) < psi)
-        return reshape_sampled(sampled, size, self.shape)
+        return g * (np.random.random(np.squeeze(g.shape)) < psi)
 
     def logp(self, value):
         psi = self.psi
         theta = self.theta
 
-        logp_val = tt.switch(value > 0,
-                     tt.log(psi) + self.pois.logp(value),
-                     logaddexp(tt.log1p(-psi), tt.log(psi) - theta))
+        logp_val = tt.switch(
+            tt.gt(value, 0),
+            tt.log(psi) + self.pois.logp(value),
+            logaddexp(tt.log1p(-psi), tt.log(psi) - theta))
 
-        return bound(logp_val,
+        return bound(
+            logp_val,
             0 <= value,
             0 <= psi, psi <= 1,
             0 <= theta)
@@ -647,7 +869,8 @@ class ZeroInflatedPoisson(Discrete):
             dist = self
         theta = dist.theta
         psi = dist.psi
-        return r'${} \sim \text{{ZeroInflatedPoisson}}(\mathit{{theta}}={}, \mathit{{psi}}={})$'.format(name,
+        name = r'\text{%s}' % name
+        return r'${} \sim \text{{ZeroInflatedPoisson}}(\mathit{{theta}}={},~\mathit{{psi}}={})$'.format(name,
                                                 get_variable_name(theta),
                                                 get_variable_name(psi))
 
@@ -656,6 +879,8 @@ class ZeroInflatedBinomial(Discrete):
     R"""
     Zero-inflated Binomial log-likelihood.
 
+    The pmf of this distribution is
+
     .. math::
 
         f(x \mid \psi, n, p) = \left\{ \begin{array}{l}
@@ -663,6 +888,27 @@ class ZeroInflatedBinomial(Discrete):
             \psi {n \choose x} p^x (1-p)^{n-x}, \text{if } x=1,2,3,\ldots,n
             \end{array} \right.
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        plt.style.use('seaborn-darkgrid')
+        x = np.arange(0, 25)
+        ns = [10, 20]
+        ps = [0.5, 0.7]
+        psis = [0.7, 0.4]
+        for n, p, psi in zip(ns, ps, psis):
+            pmf = st.binom.pmf(x, n, p)
+            pmf[0] = (1 - psi) + pmf[0]
+            pmf[1:] =  psi * pmf[1:]
+            pmf /= pmf.sum()
+            plt.plot(x, pmf, '-o', label='n = {}, p = {}, $\\psi$ = {}'.format(n, p, psi))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.legend(loc=1)
+        plt.show()
+
     ========  ==========================
     Support   :math:`x \in \mathbb{N}_0`
     Mean      :math:`(1 - \psi) n p`
@@ -688,24 +934,25 @@ class ZeroInflatedBinomial(Discrete):
         self.bin = Binomial.dist(n, p)
         self.mode = self.bin.mode
 
-    def random(self, point=None, size=None, repeat=None):
-        n, p, psi = draw_values([self.n, self.p, self.psi], point=point)
+    def random(self, point=None, size=None):
+        n, p, psi = draw_values([self.n, self.p, self.psi], point=point, size=size)
         g = generate_samples(stats.binom.rvs, n, p,
                              dist_shape=self.shape,
                              size=size)
-        sampled = g * (np.random.random(np.squeeze(g.shape)) < psi)
-        return reshape_sampled(sampled, size, self.shape)
+        return g * (np.random.random(np.squeeze(g.shape)) < psi)
 
     def logp(self, value):
         psi = self.psi
         p = self.p
         n = self.n
 
-        logp_val = tt.switch(value > 0,
-                 tt.log(psi) + self.bin.logp(value),
-                 logaddexp(tt.log1p(-psi), tt.log(psi) + n * tt.log1p(-p)))
+        logp_val = tt.switch(
+            tt.gt(value, 0),
+            tt.log(psi) + self.bin.logp(value),
+            logaddexp(tt.log1p(-psi), tt.log(psi) + n * tt.log1p(-p)))
 
-        return bound(logp_val,
+        return bound(
+            logp_val,
             0 <= value, value <= n,
             0 <= psi, psi <= 1,
             0 <= p, p <= 1)
@@ -716,10 +963,15 @@ class ZeroInflatedBinomial(Discrete):
         n = dist.n
         p = dist.p
         psi = dist.psi
-        return r'${} \sim \text{{ZeroInflatedBinomial}}(\mathit{{n}}={}, \mathit{{p}}={}, \mathit{{psi}}={})$'.format(name,
-                                                get_variable_name(n),
-                                                get_variable_name(p),
-                                                get_variable_name(psi))
+
+        name_n = get_variable_name(n)
+        name_p = get_variable_name(p)
+        name_psi = get_variable_name(psi)
+        name = r'\text{%s}' % name
+        return (r'${} \sim \text{{ZeroInflatedBinomial}}'
+                r'(\mathit{{n}}={},~\mathit{{p}}={},~'
+                r'\mathit{{psi}}={})$'
+                .format(name, name_n, name_p, name_psi))
 
 
 class ZeroInflatedNegativeBinomial(Discrete):
@@ -729,13 +981,49 @@ class ZeroInflatedNegativeBinomial(Discrete):
     The Zero-inflated version of the Negative Binomial (NB).
     The NB distribution describes a Poisson random variable
     whose rate parameter is gamma distributed.
+    The pmf of this distribution is
 
     .. math::
 
-       f(x \mid \psi, \mu, \alpha) = \left\{ \begin{array}{l}
-            (1-\psi) + \psi \left (\frac{\alpha}{\alpha+\mu} \right) ^\alpha, \text{if } x = 0 \\
-            \psi \frac{\Gamma(x+\alpha)}{x! \Gamma(\alpha)} \left (\frac{\alpha}{\mu+\alpha} \right)^\alpha \left( \frac{\mu}{\mu+\alpha} \right)^x, \text{if } x=1,2,3,\ldots
-            \end{array} \right.
+       f(x \mid \psi, \mu, \alpha) = \left\{
+         \begin{array}{l}
+           (1-\psi) + \psi \left (
+             \frac{\alpha}{\alpha+\mu}
+           \right) ^\alpha, \text{if } x = 0 \\
+           \psi \frac{\Gamma(x+\alpha)}{x! \Gamma(\alpha)} \left (
+             \frac{\alpha}{\mu+\alpha}
+           \right)^\alpha \left(
+             \frac{\mu}{\mu+\alpha}
+           \right)^x, \text{if } x=1,2,3,\ldots
+         \end{array}
+       \right.
+
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        from scipy import special
+        plt.style.use('seaborn-darkgrid')
+
+        def ZeroInfNegBinom(a, m, psi, x):
+            pmf = special.binom(x + a - 1, x) * (a / (m + a))**a * (m / (m + a))**x
+            pmf[0] = (1 - psi) + pmf[0]
+            pmf[1:] =  psi * pmf[1:]
+            pmf /= pmf.sum()
+            return pmf
+
+        x = np.arange(0, 25)
+        alphas = [2, 4]
+        mus = [2, 8]
+        psis = [0.7, 0.7]
+        for a, m, psi in zip(alphas, mus, psis):
+            pmf = ZeroInfNegBinom(a, m, psi, x)
+            plt.plot(x, pmf, '-o', label=r'$\alpha$ = {}, $\mu$ = {}, $\psi$ = {}'.format(a, m, psi))
+        plt.xlabel('x', fontsize=12)
+        plt.ylabel('f(x)', fontsize=12)
+        plt.legend(loc=1)
+        plt.show()
 
     ========  ==========================
     Support   :math:`x \in \mathbb{N}_0`
@@ -762,29 +1050,35 @@ class ZeroInflatedNegativeBinomial(Discrete):
         self.nb = NegativeBinomial.dist(mu, alpha)
         self.mode = self.nb.mode
 
-    def random(self, point=None, size=None, repeat=None):
+    def random(self, point=None, size=None):
         mu, alpha, psi = draw_values(
-            [self.mu, self.alpha, self.psi], point=point)
+            [self.mu, self.alpha, self.psi], point=point, size=size)
         g = generate_samples(stats.gamma.rvs, alpha, scale=mu / alpha,
                              dist_shape=self.shape,
                              size=size)
         g[g == 0] = np.finfo(float).eps  # Just in case
-        sampled = stats.poisson.rvs(g) * (np.random.random(np.squeeze(g.shape)) < psi)
-        return reshape_sampled(sampled, size, self.shape)
+        return stats.poisson.rvs(g) * (np.random.random(np.squeeze(g.shape)) < psi)
 
     def logp(self, value):
         alpha = self.alpha
         mu = self.mu
         psi = self.psi
 
-        logp_val = tt.switch(value > 0,
-                     tt.log(psi) + self.nb.logp(value),
-                     logaddexp(tt.log1p(-psi), tt.log(psi) + alpha * (tt.log(alpha) - tt.log(alpha + mu))))
+        logp_other = tt.log(psi) + self.nb.logp(value)
+        logp_0 = logaddexp(
+            tt.log1p(-psi),
+            tt.log(psi) + alpha * (tt.log(alpha) - tt.log(alpha + mu)))
 
-        return bound(logp_val,
-                    0 <= value,
-                    0 <= psi, psi <= 1,
-                    mu > 0, alpha > 0)
+        logp_val = tt.switch(
+            tt.gt(value, 0),
+            logp_other,
+            logp_0)
+
+        return bound(
+            logp_val,
+            0 <= value,
+            0 <= psi, psi <= 1,
+            mu > 0, alpha > 0)
 
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
@@ -792,7 +1086,101 @@ class ZeroInflatedNegativeBinomial(Discrete):
         mu = dist.mu
         alpha = dist.alpha
         psi = dist.psi
-        return r'${} \sim \text{{ZeroInflatedNegativeBinomial}}(\mathit{{mu}}={}, \mathit{{alpha}}={}, \mathit{{psi}}={})$'.format(name,
-                                                get_variable_name(mu),
-                                                get_variable_name(alpha),
-                                                get_variable_name(psi))
+
+        name_mu = get_variable_name(mu)
+        name_alpha = get_variable_name(alpha)
+        name_psi = get_variable_name(psi)
+        name = r'\text{%s}' % name
+        return (r'${} \sim \text{{ZeroInflatedNegativeBinomial}}'
+                r'(\mathit{{mu}}={},~\mathit{{alpha}}={},~'
+                r'\mathit{{psi}}={})$'
+                .format(name, name_mu, name_alpha, name_psi))
+
+
+class OrderedLogistic(Categorical):
+    R"""
+    Ordered Logistic log-likelihood.
+
+    Useful for regression on ordinal data values whose values range
+    from 1 to K as a function of some predictor, :math:`\eta`. The
+    cutpoints, :math:`c`, separate which ranges of :math:`\eta` are
+    mapped to which of the K observed dependent variables.  The number
+    of cutpoints is K - 1.  It is recommended that the cutpoints are
+    constrained to be ordered.
+
+    .. math::
+
+       f(k \mid \eta, c) = \left\{
+         \begin{array}{l}
+           1 - \text{logit}^{-1}(\eta - c_1)
+             \,, \text{if } k = 0 \\
+           \text{logit}^{-1}(\eta - c_{k - 1}) -
+           \text{logit}^{-1}(\eta - c_{k})
+             \,, \text{if } 0 < k < K \\
+           \text{logit}^{-1}(\eta - c_{K - 1})
+             \,, \text{if } k = K \\
+         \end{array}
+       \right.
+
+    Parameters
+    ----------
+    eta : float
+        The predictor.
+    c : array
+        The length K - 1 array of cutpoints which break :math:`\eta` into
+        ranges.  Do not explicitly set the first and last elements of
+        :math:`c` to negative and positive infinity.
+
+    Example
+    --------
+    .. code:: python
+
+        # Generate data for a simple 1 dimensional example problem
+        n1_c = 300; n2_c = 300; n3_c = 300
+        cluster1 = np.random.randn(n1_c) + -1
+        cluster2 = np.random.randn(n2_c) + 0
+        cluster3 = np.random.randn(n3_c) + 2
+
+        x = np.concatenate((cluster1, cluster2, cluster3))
+        y = np.concatenate((1*np.ones(n1_c),
+                            2*np.ones(n2_c),
+                            3*np.ones(n3_c))) - 1
+
+        # Ordered logistic regression
+        with pm.Model() as model:
+            cutpoints = pm.Normal("cutpoints", mu=[-1,1], sd=10, shape=2,
+                                  transform=pm.distributions.transforms.ordered)
+            y_ = pm.OrderedLogistic("y", cutpoints=cutpoints, eta=x, observed=y)
+            tr = pm.sample(1000)
+
+        # Plot the results
+        plt.hist(cluster1, 30, alpha=0.5);
+        plt.hist(cluster2, 30, alpha=0.5);
+        plt.hist(cluster3, 30, alpha=0.5);
+        plt.hist(tr["cutpoints"][:,0], 80, alpha=0.2, color='k');
+        plt.hist(tr["cutpoints"][:,1], 80, alpha=0.2, color='k');
+
+    """
+
+    def __init__(self, eta, cutpoints, *args, **kwargs):
+        self.eta = tt.as_tensor_variable(eta)
+        self.cutpoints = tt.as_tensor_variable(cutpoints)
+
+        pa = sigmoid(tt.shape_padleft(self.cutpoints) - tt.shape_padright(self.eta))
+        p_cum = tt.concatenate([
+            tt.zeros_like(tt.shape_padright(pa[:, 0])),
+            pa,
+            tt.ones_like(tt.shape_padright(pa[:, 0]))
+        ], axis=1)
+        p = p_cum[:, 1:] - p_cum[:, :-1]
+
+        super(OrderedLogistic, self).__init__(p=p, *args, **kwargs)
+
+    def _repr_latex_(self, name=None, dist=None):
+        if dist is None:
+            dist = self
+        name_eta = get_variable_name(dist.eta)
+        name_cutpoints = get_variable_name(dist.cutpoints)
+        return (r'${} \sim \text{{OrderedLogistic}}'
+                r'(\mathit{{eta}}={}, \mathit{{cutpoints}}={}$'
+                .format(name, name_eta, name_cutpoints))
