@@ -3,6 +3,7 @@ from .compound import CompoundStep
 from .hmc import HamiltonianMC, NUTS
 
 from .metropolis import Metropolis
+from .metropolis import DEMetropolis
 from .metropolis import BinaryMetropolis
 from .metropolis import BinaryGibbsMetropolis
 from .metropolis import CategoricalGibbsMetropolis
@@ -12,6 +13,7 @@ from .metropolis import LaplaceProposal
 from .metropolis import PoissonProposal
 from .metropolis import MultivariateNormalProposal
 
+from .sgmcmc import SGFS, CSG
 from .gibbs import ElemwiseCategorical
 
 from .slicer import Slice
@@ -19,4 +21,3 @@ from .slicer import Slice
 from .elliptical_slice import EllipticalSlice
 
 from .smc import SMC
-
