@@ -112,7 +112,7 @@ class BaseHMC(arraystep.GradientSharedStep):
         start = self.integrator.compute_state(q0, p0)
 
         if not np.isfinite(start.energy):
-            self.potential.raise_ok()
+            self.potential.raise_ok(self._logp_dlogp_func._ordering.vmap)
             raise ValueError('Bad initial energy: %s. The model '
                              'might be misspecified.' % start.energy)
 
@@ -164,37 +164,27 @@ class BaseHMC(arraystep.GradientSharedStep):
         self.tune = True
         self.potential.reset()
 
-    def warnings(self, strace):
+    def warnings(self):
         # list.copy() is not available in python2
         warnings = self._warnings[:]
 
         # Generate a global warning for divergences
+        message = ''
         n_divs = self._num_divs_sample
         if n_divs and self._samples_after_tune == n_divs:
-            msg = ('The chain contains only diverging samples. The model is '
-                   'probably misspecified.')
-            warning = SamplerWarning(
-                WarningType.DIVERGENCES, msg, 'error', None, None, None)
-            warnings.append(warning)
-        elif n_divs > 0:
+            message = ('The chain contains only diverging samples. The model '
+                       'is probably misspecified.')
+        elif n_divs == 1:
+            message = ('There was 1 divergence after tuning. Increase '
+                       '`target_accept` or reparameterize.')
+        elif n_divs > 1:
             message = ('There were %s divergences after tuning. Increase '
-                       '`target_accept` or reparameterize.'
-                       % n_divs)
+                       '`target_accept` or reparameterize.' % n_divs)
+
+        if message:
             warning = SamplerWarning(
                 WarningType.DIVERGENCES, message, 'error', None, None, None)
             warnings.append(warning)
 
-        # small trace
-        if self._samples_after_tune == 0:
-            msg = "Tuning was enabled throughout the whole trace."
-            warning = SamplerWarning(
-                WarningType.BAD_PARAMS, msg, 'error', None, None, None)
-            warnings.append(warning)
-        elif self._samples_after_tune < 500:
-            msg = "Only %s samples in chain." % self._samples_after_tune
-            warning = SamplerWarning(
-                WarningType.BAD_PARAMS, msg, 'error', None, None, None)
-            warnings.append(warning)
-
         warnings.extend(self.step_adapt.warnings())
         return warnings
